package csci1902.hw6.ai;

import java.util.Collection;
import java.util.Iterator;

import csci1902.hw6.game.CellPosition;
import csci1902.hw6.game.PlayerID;

class AIEngine {
/** The decay applied to scores to prefer long games. */
private static final double DEPTH_DECAY = 0.95;
/** The player this AI plays as */
private PlayerID player;
/** The depth cut-off for the game tree search */
private int depthCutOff;
/** The heuristic function used to score game boards. */
private BoardHeuristic heuristic;
/**
* Creates a new AI engine.
* @param player The player this AI will play as.
* @param heuristic The heuristic function to use for evaluating boards.
* @param depthCutOff The maximum depth to search for moves.
*/
public AIEngine(PlayerID player, BoardHeuristic heuristic, int depthCutOff) {
	this.player = player;
	this.heuristic = heuristic;
	this.depthCutOff = depthCutOff;
}
/**
* Searches for and returns the next move the AI wants to make.
* @param node The node from which the AI should move.
* @return The move made by the AI.
*/
public CellPosition getNextMove(TreeNode node) {
	double bestScore = Double.NEGATIVE_INFINITY;
	CellPosition best = null;
	Collection<CellPosition> moves = node.getSuccessorMoves();
	for(CellPosition besto: moves){
		if(getTreeScore(1, node.getChild(besto))>bestScore){
			bestScore = getTreeScore(1, node.getChild(besto));
			best = besto;
		}
	}
	return best;
}
/**
* Helper method for determining the score of a node. The meat of the
* minimax algorithm is in this method.
* @param depth The current search depth.
* @param node The node from which to search.
* @return The score of the node. If {@code depth} is equal to
* depthCutOff, this is the heuristic value of the node; otherwise, it is
* the result of appropriately recursively searching for a score.
*/
double getTreeScore(int depth, TreeNode node) {
	return heuristic.computeScore(node.getBoard(), player);
}
}