/* $Id: AIEngineTest.java,v 1.3 2007/11/28 20:25:32 ekstrand Exp $ */
package csci1902.hw6.ai;

import csci1902.hw6.game.Board;
import csci1902.hw6.game.CellPosition;
import csci1902.hw6.game.PlayerID;
import junit.framework.TestCase;

/**
 * Test cases for the AI engine.
 * @author Michael Ekstrand <ekstrand@cs.umn.edu>
 *
 */
public class AIEngineTest extends TestCase {

    private TreeNode root;
    private Board board;
    private AIEngine ai;
    
    @Override
    protected void setUp() {
        root = new TreeNode(PlayerID.X, 3, 6, 3);
        board = root.getBoard();
        ai = new AIEngine(PlayerID.X, new SimpleHeuristic(), 4);
    }
    
    /**
     * Simple test to make sure that the AI moves somewhere.
     */
    public void testGetAMove() {
        CellPosition move = ai.getNextMove(root);
        assertNotNull(move);
        assertNull(board.getCellValue(move));
        for (CellPosition m: board.getAvailableMoves()) {
            if (board.getEquivalentMoves(m).contains(move))
                return;
        }
        fail("Move not found in valid moves in board");
    }
    
    /**
     * Make sure the AI tries to win.
     */
    public void testSimpleWin() {
        // X move to 0,0
        TreeNode node = root.getChild(board.makeCellPosition(0, 0));
        // O move to 1,3
        node = node.getChild(board.makeCellPosition(1, 3));
        // X move to 1,0
        node = node.getChild(board.makeCellPosition(1, 0));
        // O move to 2,5
        node = node.getChild(board.makeCellPosition(2, 5));
        
        CellPosition move = board.makeCellPosition(2,0);
        assertEquals(move, ai.getNextMove(node));
    }
    
    /**
     * Make sure the AI tries to avoid losing.
     */
    public void testSimpleAvoidLoss() {
        // X move to 1,3
        TreeNode node = root.getChild(board.makeCellPosition(1, 3));
        // O move to 0,0
        node = node.getChild(board.makeCellPosition(0, 0));
        // X move to 2,5
        node = node.getChild(board.makeCellPosition(2, 5));
        // O move to 1,0
        node = node.getChild(board.makeCellPosition(1, 0));
        
        CellPosition move = board.makeCellPosition(2,0);
        assertEquals(move, ai.getNextMove(node));
    }

    /**
     * Make sure the AI tries to win even if there are loss-blockers.
     */
    public void testWinWithBlockers() {
        // X move to 0,1
        TreeNode node = root.getChild(board.makeCellPosition(0,1));
        // O move to 0,0
        node = node.getChild(board.makeCellPosition(0,0));
        // X to 1,2
        node = node.getChild(board.makeCellPosition(1,2));
        // O to 1,0
        node = node.getChild(board.makeCellPosition(1,0));

        // expect X to 2,3
        CellPosition move = board.makeCellPosition(2,3);
        assertEquals(move, ai.getNextMove(node));
    }
}
