/* $Id: SimpleHeuristic.java,v 1.2 2007/11/28 17:49:42 ekstrand Exp $ */

package csci1902.hw6.ai;

import csci1902.hw6.game.Board;
import csci1902.hw6.game.PlayerID;

/**
 * Simple heuristic scoring function for boards.  It simply returns the number
 * of times the player has 2 in a row, or +/- 1000 if the game ends in a win or
 * loss.
 * @author Michael Ekstrand <ekstrand@cs.umn.edu>
 *
 */
public class SimpleHeuristic implements BoardHeuristic {

    public double computeScore(Board board, PlayerID player) {
        // handle game-over condition
        if (board.isFinished()) {
            PlayerID winner = board.getWinner();
            if (winner == null)
                return 0;
            else if (winner.equals(player))
                return 1000;
            else
                return -1000;
        }
        
        double score = 0;
        
        int rows = board.getRows();
        int columns = board.getColumns();
        
        for (int r = 0; r < rows; r++) {
            for (int c = 0; c < columns; c++) {
                PlayerID p = board.getCellValue(r, c);
                if (p == null)
                    continue;
                
                boolean found = false;
                if (p.equals(board.getCellValue(r, (c+1)%columns))) {
                    found = true;       // same row
                } else if (r < rows - 1) {
                    if (p.equals(board.getCellValue(r+1, c)))
                        found = true;   // same column
                    else if (p.equals(board.getCellValue(r+1, (c+1)%columns)))
                        found = true;   // lower-right diagonal
                    else if (p.equals(board.getCellValue(r+1, (c+8-1)%columns)))
                        found = true;   // lower-left diagonal
                }
                if (found) {
                    if (p.equals(player))
                        score += 1;
                    else
                        score -= 1;
                }
            }
        }
        
        return score;
    }

}
