/* $Id: BoardTest.java,v 1.4 2007/11/28 17:49:42 ekstrand Exp $ */

package csci1902.hw6.game;

import junit.framework.TestCase;

public class BoardTest extends TestCase {
    private Board board;
    
    protected void setUp() {
        board = new Board();
    }
    
    public void testInitial() {
        assertEquals(4, board.getRows());
        assertEquals(8, board.getColumns());
        assertEquals(4, board.getGoal());
        
        for (int r = 0; r < board.getRows(); r++) {
            for (int c = 0; c < board.getColumns(); c++) {
                assertNull(board.getCellValue(board.makeCellPosition(r,c)));
                assertNull(board.getCellValue(r,c));
            }
        }
    }
    
    public void testPlay() {
        Board b2 = new Board(board, PlayerID.X, board.makeCellPosition(2,3));
        assertEquals(board.getColumns(), b2.getColumns());
        assertEquals(board.getRows(), b2.getRows());
        assertEquals(board.getGoal(), b2.getGoal());
        
        for (int r = 0; r < board.getRows(); r++) {
            for (int c = 0; c < board.getColumns(); c++) {
                if (r == 2 && c == 3) {
                    assertEquals(PlayerID.X, b2.getCellValue(board.makeCellPosition(r,c)));
                    assertEquals(PlayerID.X, b2.getCellValue(r,c));
                } else {
                    assertNull(b2.getCellValue(board.makeCellPosition(r,c)));
                    assertNull(b2.getCellValue(r,c));
                }
                assertNull(board.getCellValue(r,c));
            }
        }
        
        assertFalse(b2.isDraw());
        assertNull(b2.getWinner());
        assertFalse(b2.isFinished());
    }
    
    public void testSimpleWin() {
        Board b2 = new Board(board, PlayerID.X, board.makeCellPosition(1,3));
        b2 = new Board(b2, PlayerID.X, board.makeCellPosition(1,4));
        b2 = new Board(b2, PlayerID.X, board.makeCellPosition(1,5));
        b2 = new Board(b2, PlayerID.X, board.makeCellPosition(1,6));
        assertTrue(b2.isFinished());
        assertEquals(PlayerID.X, b2.getWinner());
        assertFalse(b2.isDraw());
    }
    
    public void testDraw() {
        Board b2 = board;
        for (int r = 0; r < board.getRows(); r++) {
            for (int c = 0; c < board.getColumns(); c++) {
                PlayerID player;
                if (c % 2 == 0)
                    player = PlayerID.X;
                else
                    player = PlayerID.O;
                if (r % 3 == 0)
                    player = player.otherPlayer();
                b2 = new Board(b2, player, b2.makeCellPosition(r, c));
            }
        }
        
        assertTrue(b2.isFinished());
        assertNull(b2.getWinner());
        assertTrue(b2.isDraw());
    }
    
    public void testComplexWin() {
        Board b2 = new Board(board, PlayerID.O, board.makeCellPosition(1, 0));
        b2 = new Board(b2, PlayerID.O, board.makeCellPosition(0, 1));
        b2 = new Board(b2, PlayerID.O, board.makeCellPosition(3, 6));
        b2 = new Board(b2, PlayerID.O, board.makeCellPosition(2, 7));
        
        assertEquals(PlayerID.O, b2.getWinner());
        assertTrue(b2.isFinished());
        assertFalse(b2.isDraw());
        
        b2 = new Board(b2, PlayerID.X, board.makeCellPosition(2,5));
        assertTrue(b2.isFinished());
        assertEquals(PlayerID.O, b2.getWinner());
    }
}
